@@ -12,6 +12,8 @@ use MOP4Import::Base::CLI_JSON -as_base
      [ignore_symlink => doc => "ignore symlinked templates"],
      [detail => doc => "show argument details"],
      [line_base => default => 1],
+     [debug_changes_dir => doc => "(LSP debugging only)"
+      , default => "var/debug_yatt_ls"],
      # qw/debug/,
    ];
 
@@ -43,6 +45,10 @@ use Plack::Util;
 use File::Basename;
 use File::stat;
 
+use File::Path qw(make_path);
+use File::Slurp qw(write_file);
+use Time::HiRes ();
+
 use Try::Tiny;
 
 use YATT::Lite;
@@ -184,6 +190,11 @@ sub apply_changes {
   $tmpl->{cf_mtime} = time;
   my $changed = join("\n", @$lines);
 
+  if ($ENV{DEBUG_YATT_LANGSERVER} and $self->debug_changes_dir_exists) {
+    my $destFn = $self->debug_changes_write_file($fileName, $changed);
+    print STDERR "# Wrote: $destFn\n";
+  }
+
   my LintResult $result;
 
   try {
@@ -209,6 +220,43 @@ sub apply_changes {
   ($changed, $result);
 }
 
+sub head_as_json_array {
+  my MY $self = shift;
+  my $limit = 10;
+  if ($_[0] =~ /^-(\d+)/) {
+    $limit = $1; shift;
+  }
+  use open qw(:std :locale);
+  local @ARGV = @_;
+  my @result;
+  while (<>) {
+    chomp;
+    push @result, $_;
+    last if --$limit <= 0;
+  }
+  \@result;
+}
+
+sub debug_changes_dir_exists {
+  (my MY $self) = @_;
+  -d "$self->{dir}/$self->{debug_changes_dir}";
+}
+
+sub debug_changes_write_file {
+  (my MY $self, my ($fileName, $changed)) = @_;
+
+  my $debugDir = "$self->{dir}/$self->{debug_changes_dir}";
+
+  substr($fileName, 0, length $self->{dir}) = "";
+
+  my $destFn = "$debugDir/$fileName." . Time::HiRes::time;
+  my $destDir = File::Basename::dirname($destFn);
+  unless (-d $destDir) {
+    make_path($destDir);
+  }
+  write_file($destFn, +{binmode => ':utf8'}, $changed);
+}
+
 # Z-chtholly(pts/0)% ./Lite/Inspector.pm apply_change_to_lines '["fooooo","bar","baz"]' '{"text":"xx","range":{"start":{"line":0,"character":1},"end":{"line":0,"character":2}}}'
 # [["fxxoooo","bar","baz"]]
 # Z-chtholly(pts/0)% ./Lite/Inspector.pm apply_change_to_lines '["fooooo","bar","baz"]' '{"text":"xx","range":{"start":{"line":0,"character":1},"end":{"line":0,"character":1}}}'
@@ -218,6 +266,20 @@ sub apply_changes {
 # Z-chtholly(pts/0)% ./Lite/Inspector.pm apply_change_to_lines '["fooooo","bar","baz"]' '{"text":"xx","range":{"start":{"line":0,"character":1},"end":{"line":1,"character":1}}}'
 # [["fxxar","baz"]]
 
+sub cmd_apply_all_change_to_lines {
+  (my MY $self, my $lines, my $changeList) = @_;
+  my $result = $self->apply_all_change_to_lines($lines, $changeList);
+  print $_, "\n" for @$result;
+}
+
+sub apply_all_change_to_lines {
+  (my MY $self, my $lines, my $changeList) = @_;
+  foreach my TextDocumentContentChangeEvent $change (@$changeList) {
+    $lines = $self->apply_change_to_lines($lines, $change);
+  }
+  $lines;
+}
+
 sub apply_change_to_lines {
   (my MY $self, my $lines, my TextDocumentContentChangeEvent $change) = @_;
   my Range $from = $change->{range};
@@ -226,19 +288,20 @@ sub apply_change_to_lines {
   my @pre = @{$lines}[0 .. $start->{line}-1];
   my @post = @{$lines}[$end->{line}+1 .. $#$lines];
   if ($start->{line} == $end->{line}) {
-    my $edited = $lines->[$start->{line}];
+    my @edited = $lines->[$start->{line}];
     try {
-      substr($edited
+      substr($edited[0]
              , $start->{character}, $end->{character} - $start->{character}
              , $change->{text});
+      @edited = split /\n/, $edited[0], -1;
     } catch {
       Carp::croak "failed to apply changes: "
-        . terse_dump([original => $edited
+        . terse_dump([original => $lines->[$start->{line}]
                       , start => $start->{character}
                       , len => $end->{character} - $start->{character}
                       , changed => $change->{text}]). ": $_";
     };
-    [@pre, $edited, @post];
+    [@pre, @edited, @post];
   } else {
     my ($pre_edit, $post_edit);
     try {
@@ -252,7 +315,8 @@ sub apply_change_to_lines {
                                  , end => $end->{character}]
                       , changed => $change->{text}]). ": $_";
     };
-    [@pre, $pre_edit.$change->{text}.$post_edit, @post];
+    my $edited = $pre_edit.$change->{text}.$post_edit;
+    [@pre, split(/\n/, $edited, -1), @post];
   }
 }
 
